{% extends "base.html" %}

{% block title %}{{ super() }}&mdash;{{ name }}{% endblock title %}

{% block content %}
<div class="jumbotron">
  <h1>{{ code }}</h1>
  <p>{{ description }}</p>
</div>
{% endblock content %}
